@@ -4,6 +4,7 @@
 #endif
 
 #if defined(__MINGW32__) || defined(_WIN32) || defined(WINDOWS)
+#define NOMINMAX
 # include <windows.h>
 # define ICONV_LITTLE_ENDIAN 1234
 # define ICONV_BIG_ENDIAN 4321
@@ -215,7 +216,9 @@ protected:
 };
 
 #if defined(C_ICONV)
+#if !defined (_MSC_VER)
 # include <iconv.h>
+#endif
 
 /* _Iconv implementation of _IconvBase using GNU libiconv or GLIBC iconv, for Linux and Mac OS X systems. */
 /* See also: "man iconv"
