@@ -1064,6 +1064,125 @@ void FPU_Init() {
 	FPU_FINIT();
 }
 
+static INLINE uint16_t fpu_tag_word_from_abridged(const uint8_t b) {
+	unsigned int i;
+	uint16_t r = 0;
+
+	/* yech... someone at Intel was trying to be too "clever" */
+	/* In the 8 bits they packed the valid/empty bitfield (with 8 bits reserved!) they could have just stored the 16-bit tag word instead! */
+	for (i=0;i < 8;i++) {
+		if (b & (1u << i)) {
+			/* TODO: Guessing the tag based on the FPU 80-bit value */
+			r |= TAG_Valid << (2u * i);
+		}
+		else {
+			r |= TAG_Empty << (2u * i);
+		}
+	}
+
+	return r;
+}
+
+static INLINE uint8_t fpu_tag_word_abridged(void) {
+	unsigned int i;
+	uint8_t r = 0;
+
+	for (i=0;i < 8;i++) {
+		if (fpu.tags[i] != TAG_Empty)
+			r |= 1u << i;
+	}
+
+	return r;
+}
+
+void CPU_FXSAVE(PhysPt eaa) {
+	unsigned int i;
+
+	FPU_SET_TOP(TOP);
+
+	/* Ref: [https://www.felixcloutier.com/x86/fxsave] */
+	mem_writew(eaa+0x000,fpu.cw);					/* +0x000 FPU control word */
+	mem_writew(eaa+0x002,fpu.sw);					/* +0x002 FPU status word */
+	mem_writeb(eaa+0x004,fpu_tag_word_abridged());			/* +0x004 FPU tag words, abridged to a bitfield of 1=not empty 0=empty, register order NOT from TOP */
+	mem_writeb(eaa+0x005,0x00);					/* +0x005 reserved */
+	mem_writew(eaa+0x006,0x0000);					/* +0x006 x87 FPU opcode (??) */
+	mem_writed(eaa+0x008,reg_eip);					/* +0x008 x87 FPU instruction pointer (???) */
+	mem_writew(eaa+0x00C,Segs.val[cs]);				/* +0x00C x87 FPU instruction pointer segment (???) */
+	mem_writew(eaa+0x00E,0x0000);					/* +0x00E reserved */
+	mem_writed(eaa+0x010,reg_eip);					/* +0x010 x87 FPU instruction operand (???) */
+	mem_writew(eaa+0x014,Segs.val[ds]);				/* +0x014 x87 FPU instruction operand segment (???) */
+	mem_writew(eaa+0x016,0x0000);					/* +0x016 reserved */
+	mem_writed(eaa+0x018,fpu.mxcsr);				/* +0x018 MXCSR */
+	mem_writed(eaa+0x01C,0xFFBF);					/* +0x01C MXCSR_MASK (DAZ not supported) */
+
+	/* NTS: Remember that st(i) TOP pointer is in FPU status word */
+
+	for (i=0;i < 8;i++) {
+#if C_FPU_X86
+		mem_writed(eaa+0x020+(i*16)+0,fpu.p_regs[STV(i)].m1);
+		mem_writed(eaa+0x020+(i*16)+4,fpu.p_regs[STV(i)].m2);
+		mem_writew(eaa+0x020+(i*16)+8,fpu.p_regs[STV(i)].m3);
+#elif defined(HAS_LONG_DOUBLE)
+		FPU_ST80(eaa+0x020+(i*16),STV(i));
+#else
+		FPU_ST80(eaa+0x020+(i*16),STV(i),/*&*/fpu.regs_80[STV(i)],fpu.use80[STV(i)]);
+#endif
+		mem_writed(eaa+0x020+(i*16)+0xA,0);
+		mem_writew(eaa+0x020+(i*16)+0xE,0);
+	}
+
+	if (CPU_SSE()) {
+		for (i=0;i < 8;i++) {
+			XMM_Reg &xmm = fpu.xmmreg[i];
+			mem_writed(eaa+0x0A0+(i*16)+0x0,xmm.u32[0]);
+			mem_writed(eaa+0x0A0+(i*16)+0x4,xmm.u32[1]);
+			mem_writed(eaa+0x0A0+(i*16)+0x8,xmm.u32[2]);
+			mem_writed(eaa+0x0A0+(i*16)+0xC,xmm.u32[3]);
+		}
+	}
+}
+
+void CPU_FXRSTOR(PhysPt eaa) {
+	unsigned int i;
+
+	/* Ref: [https://www.felixcloutier.com/x86/fxsave] */
+	fpu.cw = mem_readw(eaa+0x000);					/* +0x000 FPU control word */
+	fpu.sw = mem_readw(eaa+0x002);					/* +0x002 FPU status word */
+	fpu.mxcsr = mem_readd(eaa+0x018);				/* +0x018 MXCSR */
+
+	TOP = FPU_GET_TOP();
+
+	/* NTS: Remember that st(i) TOP pointer is in FPU status word */
+
+	for (i=0;i < 8;i++) {
+#if C_FPU_X86
+		fpu.p_regs[STV(i)].m1 = mem_readd(eaa+0x020+(i*16)+0);
+		fpu.p_regs[STV(i)].m2 = mem_readd(eaa+0x020+(i*16)+4);
+		fpu.p_regs[STV(i)].m3 = mem_readw(eaa+0x020+(i*16)+8);
+#elif defined(HAS_LONG_DOUBLE)
+		fpu.regs_80[STV(i)].v = FPU_FLD80(eaa+0x020+(i*16));
+#else
+		fpu.regs[STV(i)].d = FPU_FLD80(eaa+0x020+(i*16),/*&*/fpu.regs_80[STV(i)]);
+		fpu.use80[STV(i)] = true;
+#endif
+	}
+
+	{
+		uint16_t tw = fpu_tag_word_from_abridged(mem_readb(eaa+0x004));	/* +0x004 FPU tag words, abridged to a bitfield of 1=not empty 0=empty, register order NOT from TOP */
+		FPU_SetTag(tw);
+	}
+
+	if (CPU_SSE()) {
+		for (i=0;i < 8;i++) {
+			XMM_Reg &xmm = fpu.xmmreg[i];
+			xmm.u32[0] = mem_readd(eaa+0x0A0+(i*16)+0x0);
+			xmm.u32[1] = mem_readd(eaa+0x0A0+(i*16)+0x4);
+			xmm.u32[2] = mem_readd(eaa+0x0A0+(i*16)+0x8);
+			xmm.u32[3] = mem_readd(eaa+0x0A0+(i*16)+0xC);
+		}
+	}
+}
+
 #endif
 
 //save state support
