@@ -1,5 +1,5 @@
 /*
- *  Copyright (C) 2002-2020  The DOSBox Team
+ *  Copyright (C) 2002-2021  The DOSBox Team
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -33,6 +33,7 @@
 #include "mapper.h"
 #include "pic.h"
 #include "hardware.h"
+#include "logging.h"
 #include "timer.h"
 
 #define RAWBUF	1024
@@ -97,7 +98,7 @@ static struct {
 #include "midi_mt32.h"
 #endif
 
-#if C_FLUIDSYNTH || defined(WIN32)
+#if C_FLUIDSYNTH || defined(WIN32) && !defined(HX_DOS)
 #include "midi_synth.h"
 #endif
 
@@ -584,10 +585,11 @@ public:
 		Section_prop * section = static_cast<Section_prop *>(configuration);
 		const char * dev=section->Get_string("mididevice");
 		std::string fullconf = section->Get_string("midiconfig");
-#if C_FLUIDSYNTH || defined(WIN32)
+#if C_FLUIDSYNTH || defined(WIN32) && !defined(HX_DOS)
 		synthsamplerate = section->Get_int("samplerate");
 		if (synthsamplerate == 0) synthsamplerate = 44100;
 #endif
+		if (control->opt_silent) dev = "none";
 
 		/* If device = "default" go for first handler that works */
 		MidiHandler * handler;
@@ -657,6 +659,18 @@ public:
 	}
 };
 
+void MIDI_ListHandler(Program *caller, const char *name)
+{
+    if (!*name) return;
+    bool found=false;
+    for (auto *handler = handler_list; handler; handler = handler->next)
+        if (!strcasecmp(handler->GetName(), name)) {
+            handler->ListAll(caller);
+            found=true;
+            break;
+        }
+    if (!found) caller->WriteOut("MIDI handler not available - %s\n", name);
+}
 
 static MIDI* test = NULL;
 void MIDI_Destroy(Section* /*sec*/){
