@@ -51,9 +51,9 @@
 #include "build_timestamp.h"
 
 extern bool startcmd, startwait, startquiet, winautorun;
+extern bool halfwidthkana, force_conversion, showdbcs;
 extern bool dos_shell_running_program, mountwarning;
-extern bool halfwidthkana, force_conversion, gbk;
-extern bool addovl, addipx, addne2k, enableime;
+extern bool addovl, addipx, addne2k, enableime, gbk;
 extern const char* RunningProgram;
 extern int enablelfn, msgcodepage;
 extern uint16_t countryNo;
@@ -65,6 +65,7 @@ uint16_t shell_psp = 0;
 Bitu call_int2e = 0;
 
 std::string GetDOSBoxXPath(bool withexe=false);
+int Reflect_Menu(void);
 void SetIMPosition(void);
 void initRand();
 void initcodepagefont(void);
@@ -381,7 +382,7 @@ void DOS_Shell::ParseLine(char * line) {
 	uint16_t dummy,dummy2;
 	uint32_t bigdummy = 0;
 	bool append;
-	bool normalstdin  = false;	/* wether stdin/out are open on start. */
+	bool normalstdin  = false;	/* whether stdin/out are open on start. */
 	bool normalstdout = false;	/* Bug: Assumed is they are "con"      */
 	
     GetRedirection(line, &in, &out, &toc, &append);
@@ -441,9 +442,25 @@ void DOS_Shell::ParseLine(char * line) {
 		} else {
 			if (toc&&DOS_FindFirst(pipetmp, ~DOS_ATTR_VOLUME)&&!DOS_UnlinkFile(pipetmp))
 				fail=true;
-			status = DOS_OpenFileExtended(toc?pipetmp:out,OPEN_READWRITE,DOS_ATTR_ARCHIVE,0x12,&dummy,&dummy2);
+			status = DOS_OpenFileExtended(toc&&!fail?pipetmp:out,OPEN_READWRITE,DOS_ATTR_ARCHIVE,0x12,&dummy,&dummy2);
+			if (toc&&(fail||!status)&&!strchr(pipetmp,'\\')&&(Drives[0]||Drives[2])) {
+				int len = (int)strlen(pipetmp);
+				if (len > 266) {
+					len = 266;
+					pipetmp[len] = 0;
+				}
+                for (int i = len; i >= 0; i--)
+                    pipetmp[i + 3] = pipetmp[i];
+                pipetmp[0] = Drives[2]?'c':'a';
+                pipetmp[1] = ':';
+                pipetmp[2] = '\\';
+                fail=false;
+                if (DOS_FindFirst(pipetmp, ~DOS_ATTR_VOLUME) && !DOS_UnlinkFile(pipetmp))
+                    fail=true;
+                else
+                    status = DOS_OpenFileExtended(pipetmp, OPEN_READWRITE, DOS_ATTR_ARCHIVE, 0x12, &dummy, &dummy2);
+            }
 		}
-		
 		if(!status && normalstdout) {
 			DOS_OpenFile("con", OPEN_READWRITE, &dummy);							// Read only file, open con again
 			if (!toc) {
@@ -457,8 +474,7 @@ void DOS_Shell::ParseLine(char * line) {
 	/* Run the actual command */
 
 	if (this == first_shell) dos_shell_running_program = true;
-#if defined(WIN32) && !defined(C_SDL2)
-	int Reflect_Menu(void);
+#if DOSBOXMENU_TYPE == DOSBOXMENU_HMENU
 	Reflect_Menu();
 #endif
 	if (toc||(!toc&&((out&&DOS_FindDevice(out)!=DOS_FindDevice("con"))))) outcon=false;
@@ -467,8 +483,7 @@ void DOS_Shell::ParseLine(char * line) {
 	DoCommand(line);
 
 	if (this == first_shell) dos_shell_running_program = false;
-#if defined(WIN32) && !defined(C_SDL2)
-	int Reflect_Menu(void);
+#if DOSBOXMENU_TYPE == DOSBOXMENU_HMENU
 	Reflect_Menu();
 #endif
 
@@ -567,13 +582,19 @@ const char *ParseMsg(const char *msg) {
         if (real_readw(BIOSMEM_SEG,BIOSMEM_NB_COLS)>80)
             msg = str_replace(str_replace(str_replace((char *)msg, (char*)"\xBA\033[0m", (char*)"\xBA\033[0m\n"), (char*)"\xBB\033[0m", (char*)"\xBB\033[0m\n"), (char*)"\xBC\033[0m", (char*)"\xBC\033[0m\n");
         bool uselowbox = false;
-#if defined(USE_TTF)
         force_conversion = true;
         int cp=dos.loaded_codepage;
-        if (ttf.inUse && halfwidthkana && InitCodePage() && dos.loaded_codepage==932) uselowbox = true;
+        if ((showdbcs
+#if defined(USE_TTF)
+        || ttf.inUse
+#endif
+        )
+#if defined(USE_TTF)
+        && halfwidthkana
+#endif
+        && InitCodePage() && dos.loaded_codepage==932) uselowbox = true;
         force_conversion = false;
         dos.loaded_codepage=cp;
-#endif
         if (uselowbox || IS_JEGA_ARCH || IS_JDOSV) {
             std::string m=msg;
             if (strstr(msg, "\xCD\xCD\xCD\xCD") != NULL) {
@@ -1555,7 +1576,7 @@ void SHELL_Init() {
     MSG_Add("SHELL_CMD_ASSOC_HELP", "Displays or changes file extension associations.\n");
     MSG_Add("SHELL_CMD_ASSOC_HELP_LONG", "ASSOC [.ext[=command] ... ]\n\nType ASSOC without parameters to display the current file associations.\nFile extensions must start with a dot (.); wildcards (* and ?) are allowed.\n");
 	MSG_Add("SHELL_CMD_CHCP_HELP", "Displays or changes the current DOS code page.\n");
-	MSG_Add("SHELL_CMD_CHCP_HELP_LONG", "CHCP [nnn [file]]\n\n  nnn   Specifies a code page number.\n  file  Specifies a code page file.\n\nSupported code pages for changing in the TrueType font output:\n\n437,737,775,808,850,852,853,855,857,858,860-866,869,872,874\n\nWindows code pages: 1250,1251,1252,1253,1254,1255,1256,1257,1258\n\nAlso double-byte code pages including 932, 936, 949, and 950.\n");
+	MSG_Add("SHELL_CMD_CHCP_HELP_LONG", "CHCP [nnn [file]]\n\n  nnn   Specifies a code page number.\n  file  Specifies a code page file.\n\nSupported code pages for changing in the TrueType font output:\n\n437,737,775,808,850,852,853,855,857,858,860-866,869,872,874,3021\n\nWindows code pages: 1250,1251,1252,1253,1254,1255,1256,1257,1258\n\nAlso double-byte code pages including 932, 936, 949, and 950.\n\nCustomized code pages are supported by providing code page files.\n");
 	MSG_Add("SHELL_CMD_CHCP_ACTIVE", "Active code page: %d\n");
 	MSG_Add("SHELL_CMD_CHCP_MISSING", "ASCII characters not defined in TTF font: %d\n");
 	MSG_Add("SHELL_CMD_CHCP_INVALID", "Invalid code page number - %s\n");
@@ -1739,8 +1760,7 @@ void SHELL_Init() {
  * What the hell were you guys thinking? --J.C. */
 void SHELL_Run() {
 	dos_shell_running_program = false;
-#if defined(WIN32) && !defined(C_SDL2)
-	int Reflect_Menu(void);
+#if DOSBOXMENU_TYPE == DOSBOXMENU_HMENU
 	Reflect_Menu();
 #endif
 
@@ -1793,8 +1813,7 @@ void SHELL_Run() {
 		first_shell = 0;//Make clear that it shouldn't be used anymore
 		prepared = false;
 		dos_shell_running_program = false;
-#if defined(WIN32) && !defined(C_SDL2)
-		int Reflect_Menu(void);
+#if DOSBOXMENU_TYPE == DOSBOXMENU_HMENU
 		Reflect_Menu();
 #endif
 	}
@@ -1803,8 +1822,7 @@ void SHELL_Run() {
 		first_shell = 0;//Make clear that it shouldn't be used anymore
 		prepared = false;
 		dos_shell_running_program = false;
-#if defined(WIN32) && !defined(C_SDL2)
-		int Reflect_Menu(void);
+#if DOSBOXMENU_TYPE == DOSBOXMENU_HMENU
 		Reflect_Menu();
 #endif
 		throw;
